/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/wrangler/pkg/generic"
	v1 "k8s.io/api/rbac/v1"
	"k8s.io/apimachinery/pkg/api/equality"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	informers "k8s.io/client-go/informers/rbac/v1"
	clientset "k8s.io/client-go/kubernetes/typed/rbac/v1"
	listers "k8s.io/client-go/listers/rbac/v1"
	"k8s.io/client-go/tools/cache"
)

type RoleBindingHandler func(string, *v1.RoleBinding) (*v1.RoleBinding, error)

type RoleBindingController interface {
	generic.ControllerMeta
	RoleBindingClient

	OnChange(ctx context.Context, name string, sync RoleBindingHandler)
	OnRemove(ctx context.Context, name string, sync RoleBindingHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() RoleBindingCache
}

type RoleBindingClient interface {
	Create(*v1.RoleBinding) (*v1.RoleBinding, error)
	Update(*v1.RoleBinding) (*v1.RoleBinding, error)

	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.RoleBinding, error)
	List(namespace string, opts metav1.ListOptions) (*v1.RoleBindingList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.RoleBinding, err error)
}

type RoleBindingCache interface {
	Get(namespace, name string) (*v1.RoleBinding, error)
	List(namespace string, selector labels.Selector) ([]*v1.RoleBinding, error)

	AddIndexer(indexName string, indexer RoleBindingIndexer)
	GetByIndex(indexName, key string) ([]*v1.RoleBinding, error)
}

type RoleBindingIndexer func(obj *v1.RoleBinding) ([]string, error)

type roleBindingController struct {
	controllerManager *generic.ControllerManager
	clientGetter      clientset.RoleBindingsGetter
	informer          informers.RoleBindingInformer
	gvk               schema.GroupVersionKind
}

func NewRoleBindingController(gvk schema.GroupVersionKind, controllerManager *generic.ControllerManager, clientGetter clientset.RoleBindingsGetter, informer informers.RoleBindingInformer) RoleBindingController {
	return &roleBindingController{
		controllerManager: controllerManager,
		clientGetter:      clientGetter,
		informer:          informer,
		gvk:               gvk,
	}
}

func FromRoleBindingHandlerToHandler(sync RoleBindingHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.RoleBinding
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.RoleBinding))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *roleBindingController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.RoleBinding))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateRoleBindingDeepCopyOnChange(client RoleBindingClient, obj *v1.RoleBinding, handler func(obj *v1.RoleBinding) (*v1.RoleBinding, error)) (*v1.RoleBinding, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *roleBindingController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, handler)
}

func (c *roleBindingController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), handler)
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, removeHandler)
}

func (c *roleBindingController) OnChange(ctx context.Context, name string, sync RoleBindingHandler) {
	c.AddGenericHandler(ctx, name, FromRoleBindingHandlerToHandler(sync))
}

func (c *roleBindingController) OnRemove(ctx context.Context, name string, sync RoleBindingHandler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), FromRoleBindingHandlerToHandler(sync))
	c.AddGenericHandler(ctx, name, removeHandler)
}

func (c *roleBindingController) Enqueue(namespace, name string) {
	c.controllerManager.Enqueue(c.gvk, c.informer.Informer(), namespace, name)
}

func (c *roleBindingController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controllerManager.EnqueueAfter(c.gvk, c.informer.Informer(), namespace, name, duration)
}

func (c *roleBindingController) Informer() cache.SharedIndexInformer {
	return c.informer.Informer()
}

func (c *roleBindingController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *roleBindingController) Cache() RoleBindingCache {
	return &roleBindingCache{
		lister:  c.informer.Lister(),
		indexer: c.informer.Informer().GetIndexer(),
	}
}

func (c *roleBindingController) Create(obj *v1.RoleBinding) (*v1.RoleBinding, error) {
	return c.clientGetter.RoleBindings(obj.Namespace).Create(obj)
}

func (c *roleBindingController) Update(obj *v1.RoleBinding) (*v1.RoleBinding, error) {
	return c.clientGetter.RoleBindings(obj.Namespace).Update(obj)
}

func (c *roleBindingController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	return c.clientGetter.RoleBindings(namespace).Delete(name, options)
}

func (c *roleBindingController) Get(namespace, name string, options metav1.GetOptions) (*v1.RoleBinding, error) {
	return c.clientGetter.RoleBindings(namespace).Get(name, options)
}

func (c *roleBindingController) List(namespace string, opts metav1.ListOptions) (*v1.RoleBindingList, error) {
	return c.clientGetter.RoleBindings(namespace).List(opts)
}

func (c *roleBindingController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.clientGetter.RoleBindings(namespace).Watch(opts)
}

func (c *roleBindingController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.RoleBinding, err error) {
	return c.clientGetter.RoleBindings(namespace).Patch(name, pt, data, subresources...)
}

type roleBindingCache struct {
	lister  listers.RoleBindingLister
	indexer cache.Indexer
}

func (c *roleBindingCache) Get(namespace, name string) (*v1.RoleBinding, error) {
	return c.lister.RoleBindings(namespace).Get(name)
}

func (c *roleBindingCache) List(namespace string, selector labels.Selector) ([]*v1.RoleBinding, error) {
	return c.lister.RoleBindings(namespace).List(selector)
}

func (c *roleBindingCache) AddIndexer(indexName string, indexer RoleBindingIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.RoleBinding))
		},
	}))
}

func (c *roleBindingCache) GetByIndex(indexName, key string) (result []*v1.RoleBinding, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	for _, obj := range objs {
		result = append(result, obj.(*v1.RoleBinding))
	}
	return result, nil
}
